<?php

namespace tests\Service\DataWarehouse;

use Service\DataWarehouse\Manager;
use Service\DataWarehouse\ManagerInterface;
use Service\Manager\Mongo\Types\DateType;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DB\Mongo\Mongo;

class ManagerTest extends \PHPUnit_Framework_TestCase {
    const MONGO_DB_NAME = 'ZeedhiKPIUnitTest';
    const MONGO_PORT = '27017';
    const MONGO_IP = '192.168.122.55';
    const FACT_SALES_VOLUME = 'salesVolume';
    const DATE_FORMAT = 'd/m/Y H:i:s';

    /** @var ManagerInterface */
    protected $managerDW;
    /** @var Mongo */
    protected static $mongoDB;

    public static function setUpBeforeClass() {
        if (!extension_loaded('mongo')) {
            self::markTestSkipped('The ' . __CLASS__ . ' requires the use of mongo extension.');
        }
        self::$mongoDB = new Mongo(self::MONGO_IP, self::MONGO_PORT, self::MONGO_DB_NAME);
    }

    public function setUp() {
        $this->managerDW = new Manager(self::$mongoDB);
    }

    public function testAddFactWithObjectDateTime() {
        $row = array('NMRAZSOCIALORG' => 'PADRÃO',
            'NRORG' => '1',
            'CDEMPRESA' => '02',
            'NMEMPRESA' => 'MADERO MATRIZ',
            'CDAREAGERE' => null,
            'NMAREAGERE' => null,
            'CDOPERADOR' => null,
            'NMOPERADOR' => null,
            'CDFILIAL' => '0017',
            'NMFILIAL' => 'MADERO JARDIM SOCIAL',
            'CDLOJA' => '001',
            'NMLOJA' => 'LOJA PADRAO TESTE DE SISTEMA',
            'DTENTRVENDA' => \DateTime::createFromFormat('d/m/Y H:i:s', '29/07/2015 18:18:00', DateType::getUTC()),
            'META' => '0',
            'VALOR_VENDA' => '18676');
        $this->assertTrue($this->managerDW->addFact(self::FACT_SALES_VOLUME, $row));
    }

    public function testAddFactWithDateTimestamp() {
        $row = array('NMRAZSOCIALORG' => 'PADRÃO',
            'NRORG' => '1',
            'CDEMPRESA' => '02',
            'NMEMPRESA' => 'MADERO MATRIZ',
            'CDAREAGERE' => null,
            'NMAREAGERE' => null,
            'CDOPERADOR' => null,
            'NMOPERADOR' => null,
            'CDFILIAL' => '0017',
            'NMFILIAL' => 'MADERO JARDIM SOCIAL',
            'CDLOJA' => '001',
            'NMLOJA' => 'LOJA PADRAO TESTE DE SISTEMA',
            'DATA' => \DateTime::createFromFormat('d/m/Y H:i:s', '29/07/2015 18:18:00', DateType::getUTC())->getTimestamp(),
            'META' => '0',
            'VALOR_VENDA' => '18676');
        $this->assertTrue($this->managerDW->addFact(self::FACT_SALES_VOLUME, $row));
    }

    public function testAddFactWithDateString() {
        $row = array('NMRAZSOCIALORG' => 'PADRÃO',
            'NRORG' => '1',
            'CDEMPRESA' => '02',
            'NMEMPRESA' => 'MADERO MATRIZ',
            'CDAREAGERE' => null,
            'NMAREAGERE' => null,
            'CDOPERADOR' => null,
            'NMOPERADOR' => null,
            'CDFILIAL' => '0017',
            'NMFILIAL' => 'MADERO JARDIM SOCIAL',
            'CDLOJA' => '001',
            'NMLOJA' => 'LOJA PADRAO TESTE DE SISTEMA',
            'DATA' => "30/01/2015 10:20:00",
            'META' => '0',
            'VALOR_VENDA' => '18676');
        $this->assertTrue($this->managerDW->addFact(self::FACT_SALES_VOLUME, $row));
    }

    public function testFindFactWithDateString() {
        $this->addFactsToCollection();
        $filterCriteria = new FilterCriteria(self::FACT_SALES_VOLUME);
        $start = $end = '01/05/2013 00:00:00';
        $rows = $this->managerDW->findFact(self::FACT_SALES_VOLUME, $filterCriteria, $start, $end);
        $this->assertCount(5, $rows);
    }

    public function testFindFactWithDateObject() {
        $this->addFactsToCollection();
        $filterCriteria = new FilterCriteria(self::FACT_SALES_VOLUME);
        $start = $end = \DateTime::createFromFormat(self::DATE_FORMAT, '01/05/2013 00:00:00', DateType::getUTC());
        $rows = $this->managerDW->findFact(self::FACT_SALES_VOLUME, $filterCriteria, $start, $end);
        $this->assertCount(5, $rows);
    }

    protected function addFactsToCollection() {
        $ds = DIRECTORY_SEPARATOR;
        $slashes = $ds . '..' . $ds . '..' . $ds;
        $collectionPath = realpath(__DIR__) . $slashes . 'mocks' . $ds . 'collections' . $ds . 'salesVolume.json';
        $data = json_decode(file_get_contents($collectionPath), true);
        foreach ($data as $row) {
            $this->assertTrue($this->managerDW->addFact(self::FACT_SALES_VOLUME, $row));
        }
    }

    public function tearDown() {
        self::$mongoDB->dropCollection(self::FACT_SALES_VOLUME);
    }
}
