import { Metadata, Http, RestDatasource } from '@zeedhi/core';
import { Modal, ModalService, Form, Select, Password, LoadingService, AlertService, DialogService, Report } from '@zeedhi/common';
import { TekGrid } from "@zeedhi/teknisa-components-common";

export class DBConnectionController {
    //  Metodos de alert/dialog
    public showAlert(type: string, mensagem: string) {
        AlertService.show({
            name: 'custom-alert',
            text: mensagem,
            right: true,
            top: true,
            color: type
        });
    }
    public showErrorAlert() {
        this.showAlert("error", "Ocorreu um erro!");
    }
    public dialogEvents: any = {
        click: async (params: any) => {
            if (params.component.name === "sim") {
                await this.deleteRow(params);
            }
            DialogService.hide();
        }
    };
    public showDeleteDialog({ event }: any) {
        event.preventDefault();
        DialogService.show({
          name: 'delete-dialog',
          type: 'warning',
          title: ' ',
          text: 'Deseja remover o(s) registro(s) selecionado(s)?',
          buttons: [
            { name: 'sim', component: 'ZdButton', label: 'SIM', events: this.dialogEvents, outline: true },
            { name: 'não', component: 'ZdButton', label: 'NÃO', events: this.dialogEvents },
          ],
        });
    }
    public showErrorDialog(title: string, text: string) {
        DialogService.show({
            name: 'error-dialog',
            type: 'error',
            title,
            text
        });
    }
    public showSuccessDialog(title: string, text: string) {
        DialogService.show({
            name: 'success-dialog',
            type: 'success',
            title,
            text
        });
    }
    public showWarningDialog(title: string, text: string) {
        DialogService.show({
            name: 'warning-dialog',
            type: 'warning',
            title,
            text
        });
    }
    //  Metodos de Modal
    private detailModal!: Modal;
    public async createDetailModal() {
        this.detailModal = await ModalService.createFromJson('dbConnection/DetailModal/DetailModal', true);
    }
    public destroyDetailModal() {
        this.detailModal?.destroy();
    }
    public closeDetailModal() {
        this.detailModal.hide();
    }
    //  Metodos do Form
    protected modalData: any;
    public get currentRowId() {
        return this.modalData._id;
    }
    public get modalTitle() {
        return (this.currentRowId ? "Editar" : "Adicionar") + " conexão";
    }
    public get modalSaveButtonText() {
        return this.currentRowId ? "Salvar" : "Adicionar";
    }
    public get formData() {
        const form = this.detailModal.getChildInstance<Form>("detailForm");
        return form.value;
    }
    public get hasWallet() {
        return this.formData.database?.name === "Oracle";
    }
    public get hasLogin() {
        const form = this.formData;
        return !(form.database?.name === "xlsx" || (form.database?.name === "Oracle" && form.oracleWalletUse));
    }
    public get dbUrlLabel() {
        return (this.formData.database?.name === "xlsx" ? "Caminho para o arquivo" : "String de conexão") + "*";
    }
    public get dburlHint() {
        return this.formData.database?.name === "xlsx" ? "Ex: c:\\excel\\results.xlsx." : "Ex: jdbc:oracle:thin:@12.0.0.1:9999/pdborcl";
    }
    public async openDetailModal({ component, row }: any) {
        this.destroyDetailModal();
        await this.createDetailModal();
        this.modalData = row === undefined ? {isActive: true, service: true} : row;
        this.detailModal.show();
    }
    public modalMounted() {
        const dbAuthForm: Form = Metadata.getInstance<Form>("dbAuthForm");
        const dbAuthValue = dbAuthForm.value;
        const form: Form = this.detailModal.getChildInstance<Form>("detailForm");
        const formValue = this.formData;
        const modalData = this.modalData;
        //  Form Values
        formValue.name = modalData.name;
        formValue.main = modalData.main;
        formValue.oracleWalletUse = modalData.oracleWalletUse;
        formValue.isActive = modalData.isActive;
        formValue.dbUrl = modalData.dbUrl;
        formValue.room = modalData.room;
        form.getChildInstance<Select>('database').setValue(modalData.database);
        form.getChildInstance<Select>('language').setValue(modalData.language);
        //  Auth Form Values
        dbAuthValue.username = modalData.username;
        dbAuthForm.getChildInstance<Password>('password').appendIcon = "";
        dbAuthValue.password = modalData.password;
        dbAuthValue.service = modalData.service;
    }
    public get requestData() {
        const dbAuthForm = Metadata.getInstance<Form>("dbAuthForm").value;
        const values = this.formData;
        const content = {
            ...values,
            ...dbAuthForm,
            ...values.database,
            ...values.language,
            languageLabel: values.language.label,
            passwordNeedEncrypt: dbAuthForm.password !== this.modalData.passwordSsl,
            name: values.name,
            _id: this.currentRowId,
            passwordSsl: this.modalData.passwordSsl,
            "changed": true
        };
        if (!content.passwordNeedEncrypt) {
            delete content.passwordNeedEncrypt;
        }
        if (!this.hasWallet) {
            content.oracleWalletUse = false;
        }
        if (!this.hasLogin) {
            content.username = null;
            content.usernameSsl = null;
            content.password = null;
            content.passwordSsl = null;
            content.service = false;
        }
        return content;
    }
    public validateRequestData() {
        const form = this.detailModal.getChildInstance<Form>("detailForm");
        const dbAuthForm = Metadata.getInstance<Form>("dbAuthForm");
        const formValid = form.validate();
        const formAuthValid = this.hasLogin ? dbAuthForm.validate() : true;
        return formValid && formAuthValid;
    }
    public validateWallet({dbUrl, oracleWalletUse}: any) {
        const walletValidationRegex = /USR_ORG_(\d*)/g;
        if (oracleWalletUse && dbUrl.match(walletValidationRegex)) {
            const regexRes = walletValidationRegex.exec(dbUrl);
            const wallet = regexRes ? regexRes[1] : null;
            if (wallet !== this.WALLET_NRORG) {
                this.showWarningDialog("Atenção", "Wallet fora do padrão de segurança! Por motivos de segurança, o Wallet deve não conter 'USR_ORG_' exeto se for 'USR_ORG_" + this.WALLET_NRORG + "'");
                return false;
            }
        }
        return true;
    }
    public async saveDetailModal() {
        if (!this.validateRequestData()) {
            return;
        }
        const content = this.requestData;
        if (content.oracleWalletUse === true || content.oracleWalletUse === "Y") {
            content.username = "";
            content.password = "";
            content.usernameSsl = "";
            content.passwordSsl = "";
        }

        if (this.hasWallet && !this.validateWallet(content)) {
            return;
        }
        const addContent = {
            __createdLocal: true,
            __is_new: true,
            ...content
        };
        //  Seguindo o funcionamento da tela, apenas as conexões a MySQL e xlsx são testadas antes de salvar, e caso falhem não salvam
        if ((content.database === "MySQL" || content.database === "xlsx") && !(await this.testConnection())) {
            return;
        }
        try {
            const res = await Http.post("connection/save", {
                "requestType": "DataSet",
                "disableLoader": false,
                "dataset": [this.currentRowId ? content : addContent]
            });
            this.showAlert('success', "Salvo com sucesso!");
        } catch (error) {
            console.error(error);
            this.showErrorAlert();
        }
        this.reloadGrid();
        this.closeDetailModal();
    }
    public async testConnection() {
        if (!this.validateRequestData()) {
            return false;
        }
        const content = this.requestData;
        if (this.hasWallet && !this.validateWallet(content)) {
            return false;
        }
        let status = false;
        LoadingService.show("Aguarde...");
        try {
            const res = await Http.post("testConnection", {
                "requestType": "Row",
                "row": content
            });
            const data = res.data.dataset.testResponse;
            status = !data.hasError;
            if (!status) {
                this.showErrorDialog("Ocorreu um erro", data.response);
            } else {
                this.showSuccessDialog("Sucesso", "Conectado com sucesso" + (data.response.version ? " com o coletor na versão " + data.response.version : "!"));
            }
        } catch (error) {
            console.error(error);
            status = false;
            this.showErrorAlert();
        }
        LoadingService.hide();
        return status;
    }
    //  Metodos do Grid
    public WALLET_NRORG: any;
    public get visibleColumns() {
        const grids = Metadata.getInstances('connection-tek-grid');
        if (grids.length > 0 && grids[0]) {
            const grid = grids[0];
            return grid.columns.filter((column: any) => column.isVisible).map((column: any) => column.name);
        }
        return [];
    }
    public async getReport({ component }: any) {
        const grid = Metadata.getInstance<TekGrid>('connection-tek-grid');
        const report = new Report(grid, grid.title);
        const datasourceClone = new RestDatasource(grid.datasource);
        const type = component.name;
        datasourceClone.loadAll = true;
        const data = await datasourceClone.get();
        report.data = data.connection;
        if (type === "csv") {
            const workingCSVEndPoint = "https://zhreports.teknisa.cloud";
            report.endPoint = workingCSVEndPoint;
            report.fileEndPoint = workingCSVEndPoint + "/reports/";
        }
        const reportUrl = await report.getReport(type);
        window.open(reportUrl);
    }
    public async reloadGrid() {
        const grid = Metadata.getInstance<TekGrid>('connection-tek-grid');
        return grid.reload();
    }
    public handleResponse({ component }: any) {
        const info = component.currentRow;
        if (info.connection !== undefined) {
            this.handleGridResponde(info);
        }
    }
    public handleGridResponde({connection, WALLET_NRORG, pagination}: any) {
        this.WALLET_NRORG = WALLET_NRORG[0];
        const grid = Metadata.getInstance<TekGrid>('connection-tek-grid');
        const datasource = grid.datasource;
        datasource.data = connection;
        datasource.total = pagination.total;
        datasource.limit = pagination.limit;
        datasource.page = pagination.page;
    }
    public async deleteRow({ component, event }: any) {
        const grid = Metadata.getInstance<TekGrid>('connection-tek-grid');
        event.preventDefault();
        const response = await Http.post("connection/remove", {
            "requestType": "DataSet",
            "dataset": grid.selectedRows
        }).then(() => {
            this.showAlert('success', "Deletado com sucesso!");
        }).catch((error) => {
            console.error(error);
            this.showErrorAlert();
        }).finally(() => {
            this.reloadGrid();
            grid.selectedRows = [];
        });
    }
}
